| Safe Haskell | None |
|---|---|
| Language | Haskell2010 |
Data.Automaton.Trans.RWS
Description
Synopsis
- rwsS :: forall (m :: Type -> Type) w r s a b. (Functor m, Monad m, Monoid w) => Automaton m (r, s, a) (w, s, b) -> Automaton (RWST r w s m) a b
- runRWSS :: forall (m :: Type -> Type) w r s a b. (Functor m, Monad m, Monoid w) => Automaton (RWST r w s m) a b -> Automaton m (r, s, a) (w, s, b)
- liftCallCC' :: Monoid w => CallCC m (a, s, w) (b, s, w) -> CallCC (RWST r w s m) a b
- get :: forall w (m :: Type -> Type) r s. (Monoid w, Monad m) => RWST r w s m s
- tell :: forall (m :: Type -> Type) w r s. Monad m => w -> RWST r w s m ()
- put :: forall w (m :: Type -> Type) s r. (Monoid w, Monad m) => s -> RWST r w s m ()
- ask :: forall w (m :: Type -> Type) r s. (Monoid w, Monad m) => RWST r w s m r
- asks :: forall w (m :: Type -> Type) r a s. (Monoid w, Monad m) => (r -> a) -> RWST r w s m a
- local :: forall r w s (m :: Type -> Type) a. (r -> r) -> RWST r w s m a -> RWST r w s m a
- listen :: forall (m :: Type -> Type) r w s a. Monad m => RWST r w s m a -> RWST r w s m (a, w)
- listens :: forall (m :: Type -> Type) w b r s a. Monad m => (w -> b) -> RWST r w s m a -> RWST r w s m (a, b)
- censor :: forall (m :: Type -> Type) w r s a. Monad m => (w -> w) -> RWST r w s m a -> RWST r w s m a
- newtype RWST r w s (m :: Type -> Type) a = RWST {
- runRWST :: r -> s -> m (a, s, w)
- type RWS r w s = RWST r w s Identity
- rws :: (r -> s -> (a, s, w)) -> RWS r w s a
- runRWS :: RWS r w s a -> r -> s -> (a, s, w)
- evalRWS :: RWS r w s a -> r -> s -> (a, w)
- execRWS :: RWS r w s a -> r -> s -> (s, w)
- mapRWS :: ((a, s, w) -> (b, s, w')) -> RWS r w s a -> RWS r w' s b
- withRWS :: (r' -> s -> (r, s)) -> RWS r w s a -> RWS r' w s a
- evalRWST :: Monad m => RWST r w s m a -> r -> s -> m (a, w)
- execRWST :: Monad m => RWST r w s m a -> r -> s -> m (s, w)
- mapRWST :: (m (a, s, w) -> n (b, s, w')) -> RWST r w s m a -> RWST r w' s n b
- withRWST :: forall r' s r w (m :: Type -> Type) a. (r' -> s -> (r, s)) -> RWST r w s m a -> RWST r' w s m a
- reader :: forall w (m :: Type -> Type) r a s. (Monoid w, Monad m) => (r -> a) -> RWST r w s m a
- state :: forall w (m :: Type -> Type) s a r. (Monoid w, Monad m) => (s -> (a, s)) -> RWST r w s m a
- modify :: forall w (m :: Type -> Type) s r. (Monoid w, Monad m) => (s -> s) -> RWST r w s m ()
- gets :: forall w (m :: Type -> Type) s a r. (Monoid w, Monad m) => (s -> a) -> RWST r w s m a
- writer :: forall (m :: Type -> Type) a w r s. Monad m => (a, w) -> RWST r w s m a
- pass :: forall (m :: Type -> Type) r w s a. Monad m => RWST r w s m (a, w -> w) -> RWST r w s m a
Documentation
rwsS :: forall (m :: Type -> Type) w r s a b. (Functor m, Monad m, Monoid w) => Automaton m (r, s, a) (w, s, b) -> Automaton (RWST r w s m) a b Source #
runRWSS :: forall (m :: Type -> Type) w r s a b. (Functor m, Monad m, Monoid w) => Automaton (RWST r w s m) a b -> Automaton m (r, s, a) (w, s, b) Source #
Run the RWST layer by making the state variables explicit.
liftCallCC' :: Monoid w => CallCC m (a, s, w) (b, s, w) -> CallCC (RWST r w s m) a b #
In-situ lifting of a callCC operation to the new monad.
This version uses the current state on entering the continuation.
get :: forall w (m :: Type -> Type) r s. (Monoid w, Monad m) => RWST r w s m s #
Fetch the current value of the state within the monad.
tell :: forall (m :: Type -> Type) w r s. Monad m => w -> RWST r w s m () #
is an action that produces the output tell ww.
put :: forall w (m :: Type -> Type) s r. (Monoid w, Monad m) => s -> RWST r w s m () #
sets the state within the monad to put ss.
ask :: forall w (m :: Type -> Type) r s. (Monoid w, Monad m) => RWST r w s m r #
Fetch the value of the environment.
listens :: forall (m :: Type -> Type) w b r s a. Monad m => (w -> b) -> RWST r w s m a -> RWST r w s m (a, b) #
censor :: forall (m :: Type -> Type) w r s a. Monad m => (w -> w) -> RWST r w s m a -> RWST r w s m a #
newtype RWST r w s (m :: Type -> Type) a #
A monad transformer adding reading an environment of type r,
collecting an output of type w and updating a state of type s
to an inner monad m.
Instances
| (Monoid w, Monad m) => MonadRWS r w s (RWST r w s m) | |||||
Defined in Control.Monad.RWS.Class | |||||
| MFunctor (RWST r w s :: (Type -> Type) -> Type -> Type) | |||||
| (Monoid w, MonadSplit g m) => MonadSplit g (RWST r w s m) | |||||
Defined in Control.Monad.Random.Class | |||||
| (MonadAccum w' m, Monoid w) => MonadAccum w' (RWST r w s m) | Since: mtl-2.3 | ||||
| (Monoid w, MonadError e m) => MonadError e (RWST r w s m) | |||||
Defined in Control.Monad.Error.Class Methods throwError :: e -> RWST r w s m a # catchError :: RWST r w s m a -> (e -> RWST r w s m a) -> RWST r w s m a # | |||||
| (Monad m, Monoid w) => MonadReader r (RWST r w s m) | |||||
| (MonadSelect w' m, Monoid w) => MonadSelect w' (RWST r w s m) | A combination of an 'outer' Since: mtl-2.3 | ||||
Defined in Control.Monad.Select | |||||
| (Monad m, Monoid w) => MonadState s (RWST r w s m) | |||||
| (Monoid w, Monad m) => MonadWriter w (RWST r w s m) | |||||
| Monoid w => MonadTrans (RWST r w s) | |||||
Defined in Control.Monad.Trans.RWS.Strict | |||||
| (Monoid w, MonadInterleave m) => MonadInterleave (RWST r w s m) | |||||
Defined in Control.Monad.Random.Class Methods interleave :: RWST r w s m a -> RWST r w s m a # | |||||
| (Monoid w, MonadRandom m) => MonadRandom (RWST r w s m) | |||||
Defined in Control.Monad.Random.Class | |||||
| (Monoid w, MonadFail m) => MonadFail (RWST r w s m) | |||||
Defined in Control.Monad.Trans.RWS.Strict | |||||
| (Monoid w, MonadFix m) => MonadFix (RWST r w s m) | |||||
Defined in Control.Monad.Trans.RWS.Strict | |||||
| (Monoid w, MonadIO m) => MonadIO (RWST r w s m) | |||||
Defined in Control.Monad.Trans.RWS.Strict | |||||
| Contravariant m => Contravariant (RWST r w s m) | |||||
| (Monoid w, Functor m, MonadPlus m) => Alternative (RWST r w s m) | |||||
| (Monoid w, Functor m, Monad m) => Applicative (RWST r w s m) | |||||
Defined in Control.Monad.Trans.RWS.Strict | |||||
| Functor m => Functor (RWST r w s m) | |||||
| (Monoid w, Monad m) => Monad (RWST r w s m) | |||||
| (Monoid w, MonadPlus m) => MonadPlus (RWST r w s m) | |||||
| (Monoid w, MonadCont m) => MonadCont (RWST r w s m) | |||||
| (Monoid w, PrimMonad m) => PrimMonad (RWST r w s m) | |||||
| (Monoid w, Monad m) => Selective (RWST r w s m) | |||||
| Generic (RWST r w s m a) | |||||
Defined in Control.Monad.Trans.RWS.Strict Associated Types
| |||||
| type PrimState (RWST r w s m) | |||||
Defined in Control.Monad.Primitive | |||||
| type Rep (RWST r w s m a) | |||||
Defined in Control.Monad.Trans.RWS.Strict | |||||
type RWS r w s = RWST r w s Identity #
A monad containing an environment of type r, output of type w
and an updatable state of type s.
rws :: (r -> s -> (a, s, w)) -> RWS r w s a #
Construct an RWS computation from a function.
(The inverse of runRWS.)
runRWS :: RWS r w s a -> r -> s -> (a, s, w) #
Unwrap an RWS computation as a function.
(The inverse of rws.)
Arguments
| :: RWS r w s a | RWS computation to execute |
| -> r | initial environment |
| -> s | initial value |
| -> (a, w) | final value and output |
Evaluate a computation with the given initial state and environment, returning the final value and output, discarding the final state.
Arguments
| :: RWS r w s a | RWS computation to execute |
| -> r | initial environment |
| -> s | initial value |
| -> (s, w) | final state and output |
Evaluate a computation with the given initial state and environment, returning the final state and output, discarding the final value.
Arguments
| :: Monad m | |
| => RWST r w s m a | computation to execute |
| -> r | initial environment |
| -> s | initial value |
| -> m (a, w) | computation yielding final value and output |
Evaluate a computation with the given initial state and environment, returning the final value and output, discarding the final state.
Arguments
| :: Monad m | |
| => RWST r w s m a | computation to execute |
| -> r | initial environment |
| -> s | initial value |
| -> m (s, w) | computation yielding final state and output |
Evaluate a computation with the given initial state and environment, returning the final state and output, discarding the final value.
withRWST :: forall r' s r w (m :: Type -> Type) a. (r' -> s -> (r, s)) -> RWST r w s m a -> RWST r' w s m a #
reader :: forall w (m :: Type -> Type) r a s. (Monoid w, Monad m) => (r -> a) -> RWST r w s m a #
Constructor for computations in the reader monad (equivalent to asks).
state :: forall w (m :: Type -> Type) s a r. (Monoid w, Monad m) => (s -> (a, s)) -> RWST r w s m a #
Construct a state monad computation from a state transformer function.
writer :: forall (m :: Type -> Type) a w r s. Monad m => (a, w) -> RWST r w s m a #
Construct a writer computation from a (result, output) pair.