| Safe Haskell | Safe-Inferred |
|---|---|
| Language | Haskell2010 |
Data.Stream.Recursive
Documentation
newtype Recursive (m :: Type -> Type) a Source #
A stream transformer in recursive encoding.
One step of the stream transformer performs a monadic action and results in an output and a new stream.
Constructors
| Recursive | |
Fields
| |